package com.ningdatech.pmapi.gov.controller;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import com.ningdatech.pmapi.gov.manage.GovProjectCollectionManage;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectSaveDTO;
import com.ningdatech.pmapi.gov.model.vo.GovBizProjectDetailVO;
import com.ningdatech.pmapi.gov.model.vo.GovBizProjectListVO;
import com.ningdatech.pmapi.gov.model.vo.GovProjectDictionaryVO;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zpf
 * @since 2023-08-21
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@Api(value = "GovProjectCollectionController", tags = "新项目归集")
@RequestMapping("/api/v1/gov-project-collection")
public class GovProjectCollectionController {

    private final GovProjectCollectionManage collectionManage;

    @GetMapping("/dictionary")
    @ApiOperation("字典")
    public List<GovProjectDictionaryVO> dictionary(@RequestParam(value = "type", required = false) String type) {
        return collectionManage.dictionary(type);
    }

    @GetMapping("/list")
    @ApiOperation("项目归集列表")
    public PageVo<GovBizProjectListVO> list(ProjectListReq req) {
        return collectionManage.list(req);
    }

    @GetMapping("/detail/{projId}")
    @ApiOperation("项目归集详情")
    public GovBizProjectDetailVO detail(@PathVariable String projId) {
        return collectionManage.detail(projId);
    }

    @PostMapping("/save")
    @ApiOperation("项目归集保存")
    @WebLog("项目归集保存")
    public String save(@Valid @RequestBody GovBizProjectSaveDTO saveDTO) {
        return collectionManage.save(saveDTO);
    }

    @PostMapping("/remove/{projId}")
    @ApiOperation("项目归集删除")
    @WebLog("项目归集删除")
    public String remove(@PathVariable String projId) {
        return collectionManage.remove(projId);
    }
}
