package com.ningdatech.pmapi.gov.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.gov.contants.BizProjectContant;
import com.ningdatech.pmapi.gov.convert.ProjectConvert;
import com.ningdatech.pmapi.gov.model.dto.*;
import com.ningdatech.pmapi.gov.model.entity.*;
import com.ningdatech.pmapi.gov.model.req.ProjectApplyReq;
import com.ningdatech.pmapi.gov.model.req.ProjectBaseInfoReq;
import com.ningdatech.pmapi.gov.model.req.ProjectPushReq;
import com.ningdatech.pmapi.gov.model.vo.*;
import com.ningdatech.pmapi.gov.service.*;
import com.ningdatech.pmapi.projectdeclared.utils.GenerateProjectCodeUtil;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author zpf
 * @date 2023/8/21 下午2:27
 */

@Component
@RequiredArgsConstructor
@Slf4j
public class GovProjectCollectionManage {

    private final IGovProjectDictionaryService dictionaryService;

    private final IGovBizProjectApplyService projectApplyService;

    private final IGovBizProjectBaseinfoService baseinfoService;

    private final IGovBizProjectApproveService approveService;

    private final IGovBizProjectCimplementService cimplementService;

    private final IGovBizProjectMimplementService mimplementService;

    private final IGovBizProjectProcureService procureService;

    private final GenerateProjectCodeUtil generateProjectCodeUtil;

    private final RestTemplate restTemplate;

    public List<GovProjectDictionaryVO> dictionary(String type) {

        List<GovProjectDictionary> dictionaries = dictionaryService.list(Wrappers.lambdaQuery(GovProjectDictionary.class)
                .eq(StringUtils.isNotBlank(type), GovProjectDictionary::getType, type));

        if(CollUtil.isEmpty(dictionaries)){
            return Collections.emptyList();
        }

        return dictionaries.stream().map(d -> BeanUtil.copyProperties(d,GovProjectDictionaryVO.class))
                .collect(Collectors.toList());
    }

    /**
     * 分页查 项目归集
     * @param req
     * @return
     */
    public PageVo<GovBizProjectListVO> list(ProjectListReq req) {
        Page<GovBizProjectBaseinfo> page = req.page();
        LambdaQueryWrapper<GovBizProjectBaseinfo> wrapper = Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(StringUtils.isNotBlank(req.getProjectName()),GovBizProjectBaseinfo::getBaseProjName,req.getProjectName())
                .like(StringUtils.isNotBlank(req.getBuildOrg()),GovBizProjectBaseinfo::getBaseBuildDeprt,req.getBuildOrg())
                .orderByDesc(GovBizProjectBaseinfo::getBizTime);
        baseinfoService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<String> projIds = page.getRecords().stream()
                .filter(p -> Objects.nonNull(p) && StringUtils.isNotBlank(p.getBaseProjId()))
                .map(GovBizProjectBaseinfo::getBaseProjId).collect(Collectors.toList());

        Map<String,GovBizProjectApply> applyMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(projIds)){
            List<GovBizProjectApply> applys = projectApplyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .in(GovBizProjectApply::getBaseProjId,projIds));
            applyMap = applys.stream().collect(Collectors.toMap(GovBizProjectApply::getBaseProjId,a -> a));
        }

        Map<String, GovBizProjectApply> finalApplyMap = applyMap;
        List<GovBizProjectListVO> res = page.getRecords().stream().map(p -> {
            GovBizProjectListVO vo = new GovBizProjectListVO();
            vo.setBaseAreaCode(p.getBaseAreaCode());
            vo.setBaseAreaName(p.getBaseAreaName());
            vo.setBaseBuildDeprt(p.getBaseBuildDeprt());
            vo.setBaseBuildDeprtDing(p.getBaseBuildDeprtDing());
            vo.setBaseProjId(p.getBaseProjId());
            vo.setBaseProjName(p.getBaseProjName());
            if(finalApplyMap.containsKey(p.getBaseProjId())){
                GovBizProjectApply apply = finalApplyMap.get(p.getBaseProjId());
                vo.setBaseProjDeclAmount(apply.getBaseProjDeclAmount());
                vo.setBaseProjTotalAmount(apply.getBaseProjTotalAmount());
            }
            return vo;
        }).collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 项目归集详情
     * @param projId
     * @return
     */
    public GovBizProjectDetailVO detail(String projId) {
        GovBizProjectDetailVO res = new GovBizProjectDetailVO();
        //1.先查 基本信息
        GovBizProjectBaseinfo baseinfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, projId)
                .orderByDesc(GovBizProjectBaseinfo::getBizTime)
                .last(BizConst.LIMIT_1));

        if(Objects.isNull(baseinfo)){
            return res;
        }
        res.setBaseProjId(baseinfo.getBaseProjId());
        res.setBaseProjName(baseinfo.getBaseProjName());
        res.setBaseinfo(BeanUtil.copyProperties(baseinfo, GovBizProjectBaseinfoVO.class));

        //2.再查 申报信息
        GovBizProjectApply apply = projectApplyService.getOne(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .eq(GovBizProjectApply::getBaseProjId, projId)
                .orderByDesc(GovBizProjectApply::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(apply)){
            res.setApply(BeanUtil.copyProperties(apply, GovBizProjectApplyVO.class));
        }

        //3.再查 审批信息
        GovBizProjectApprove approve = approveService.getOne(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                .eq(GovBizProjectApprove::getBaseProjId, projId)
                .orderByDesc(GovBizProjectApprove::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(approve)){
            res.setApprove(BeanUtil.copyProperties(approve, GovBizProjectApproveVO.class));
        }

        //4 对应查询 实施项目还是运维项目
        GovBizProjectCimplement cimplement = cimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                .eq(GovBizProjectCimplement::getBaseProjId, projId)
                .orderByDesc(GovBizProjectCimplement::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(cimplement)){
            res.setCimplement(BeanUtil.copyProperties(cimplement, GovBizProjectCimplementVO.class));
        }
        GovBizProjectMimplement mimplement = mimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                .eq(GovBizProjectMimplement::getBaseProjId, projId)
                .orderByDesc(GovBizProjectMimplement::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(mimplement)){
            res.setMimplement(BeanUtil.copyProperties(mimplement, GovBizProjectMimplementVO.class));
        }

        //5.最后查询采购信息
        GovBizProjectProcure procure = procureService.getOne(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                .eq(GovBizProjectProcure::getBaseProjId, projId)
                .orderByDesc(GovBizProjectProcure::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(procure)){
            res.setProcure(BeanUtil.copyProperties(procure, GovBizProjectProcureVO.class));
        }

        return res;
    }

    /**
     * 项目归集保存
     * @param saveDTO
     * @return
     */
    public String save(GovBizProjectSaveDTO saveDTO) {
        String baseProjId = saveDTO.getBaseProjId();

        //如果是新增的话 要生成 项目编号
        if(StringUtils.isBlank(baseProjId)){
            baseProjId = generateProjectCodeUtil.generateProjectCode(saveDTO);
        }

        String baseProjName = saveDTO.getBaseProjName();

        //1.保存基本信息
        GovBizProjectBaseinfoDTO baseinfo = saveDTO.getBaseinfo();
        baseinfo.setBaseProjId(baseProjId);
        baseinfo.setBaseProjName(baseProjName);
        GovBizProjectBaseinfo oldBaseInfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, baseProjId)
                .last(BizConst.LIMIT_1));

        //项目名去重
        checkName(baseProjName,baseProjId);
        GovBizProjectBaseinfo saveBase = BeanUtil.copyProperties(baseinfo,GovBizProjectBaseinfo.class);
        saveBase.setBaseProjId(baseProjId);
        saveBase.setBaseProjName(baseProjName);
        saveBase.setIsEffective(BizProjectContant.ProjectCollection.IS_EFFECTIVE);
        if(Objects.nonNull(oldBaseInfo)){
            saveBase.setId(oldBaseInfo.getId());
        }else{
            if(Objects.nonNull(saveBase.getBaseAreaCode())){
                saveBase.setBaseAreaCode(saveBase.getBaseAreaCode() + BizConst.NINE_AREA_CODE_LAST);
                saveBase.setAreaCode(saveBase.getBaseAreaCode());
            }
            saveBase.setTongTime(LocalDateTime.now());
        }
        baseinfoService.saveOrUpdate(saveBase);

        // 2.保存 申报信息
        GovBizProjectApplyDTO apply = saveDTO.getApply();
        apply.setBaseProjId(baseProjId);
        apply.setBaseProjName(baseProjName);
        if(Objects.nonNull(apply)){
            GovBizProjectApply oldApply = projectApplyService.getOne(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .eq(GovBizProjectApply::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectApply saveApply = BeanUtil.copyProperties(apply,GovBizProjectApply.class);
            saveApply.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldApply)){
                saveApply.setId(oldApply.getId());
            }else{
                if(Objects.nonNull(saveBase.getBaseAreaCode())){
                    saveApply.setAreaCode(saveBase.getBaseAreaCode());
                }
                saveApply.setTongTime(LocalDateTime.now());
            }
            projectApplyService.saveOrUpdate(saveApply);
        }

        // 3.保存 审批信息
        GovBizProjectApproveDTO approve = saveDTO.getApprove();
        approve.setBaseProjId(baseProjId);
        approve.setBaseProjName(baseProjName);
        if(Objects.nonNull(approve)){
            GovBizProjectApprove oldApprove = approveService.getOne(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                    .eq(GovBizProjectApprove::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectApprove saveApprove = BeanUtil.copyProperties(approve,GovBizProjectApprove.class);
            saveApprove.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldApprove)){
                saveApprove.setId(oldApprove.getId());
            }else{
                if(Objects.nonNull(saveBase.getBaseAreaCode())){
                    saveApprove.setAreaCode(saveBase.getBaseAreaCode());
                }
                saveApprove.setTongTime(LocalDateTime.now());
            }
            approveService.saveOrUpdate(saveApprove);
        }

        // 4.保存 建设项目 实施信息
        GovBizProjectCimplementDTO cimplement = saveDTO.getCimplement();
        cimplement.setBaseProjId(baseProjId);
        cimplement.setBaseProjName(baseProjName);
        if(Objects.nonNull(cimplement)){
            GovBizProjectCimplement oldCimplement = cimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                    .eq(GovBizProjectCimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectCimplement saveCimplement = BeanUtil.copyProperties(cimplement,GovBizProjectCimplement.class);
            saveCimplement.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldCimplement)){
                saveCimplement.setId(oldCimplement.getId());
            }else{
                if(Objects.nonNull(saveBase.getBaseAreaCode())){
                    saveCimplement.setAreaCode(saveBase.getBaseAreaCode());
                }
                saveCimplement.setTongTime(LocalDateTime.now());
            }
            cimplementService.saveOrUpdate(saveCimplement);
        }

        // 5.保存 运维项目 实施信息
        GovBizProjectMimplementDTO mimplement = saveDTO.getMimplement();
        mimplement.setBaseProjId(baseProjId);
        mimplement.setBaseProjName(baseProjName);
        if(Objects.nonNull(mimplement)){
            GovBizProjectMimplement oldMimplement = mimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                    .eq(GovBizProjectMimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectMimplement saveMimplement = BeanUtil.copyProperties(mimplement,GovBizProjectMimplement.class);
            saveMimplement.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldMimplement)){
                saveMimplement.setId(oldMimplement.getId());
            }else{
                if(Objects.nonNull(saveBase.getBaseAreaCode())){
                    saveMimplement.setAreaCode(saveBase.getBaseAreaCode());
                }
                saveMimplement.setTongTime(LocalDateTime.now());
            }
            mimplementService.saveOrUpdate(saveMimplement);
        }

        // 6.保存 采购信息
        GovBizProjectProcureDTO procure = saveDTO.getProcure();
        procure.setBaseProjId(baseProjId);
        procure.setBaseProjName(baseProjName);
        if(Objects.nonNull(procure)){
            GovBizProjectProcure oldProcure = procureService.getOne(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                    .eq(GovBizProjectProcure::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectProcure saveProcure = BeanUtil.copyProperties(procure,GovBizProjectProcure.class);
            saveProcure.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldProcure)){
                saveProcure.setId(oldProcure.getId());
            }else{
                if(Objects.nonNull(saveBase.getBaseAreaCode())){
                    saveProcure.setAreaCode(saveBase.getBaseAreaCode());
                }
                saveProcure.setTongTime(LocalDateTime.now());
            }
            procureService.saveOrUpdate(saveProcure);
        }

        //7. 推送数据
        pushProject(saveDTO);

        return BizConst.SAVE_SUCCESS;
    }

    /**
     * 删除项目归集
     * @param projId
     * @return
     */
    public String remove(String projId) {
        GovBizProjectBaseinfo baseinfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, projId)
                .last(BizConst.LIMIT_1));
        
        VUtils.isTrue(Objects.isNull(baseinfo)).throwMessage("项目不存在！");

        baseinfoService.removeById(baseinfo);
        projectApplyService.remove(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .eq(GovBizProjectApply::getBaseProjId,projId));
        approveService.remove(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                .eq(GovBizProjectApprove::getBaseProjId,projId));
        cimplementService.remove(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                .eq(GovBizProjectCimplement::getBaseProjId,projId));
        mimplementService.remove(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                .eq(GovBizProjectMimplement::getBaseProjId,projId));
        procureService.remove(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                .eq(GovBizProjectProcure::getBaseProjId,projId));
        notEffectiveProject(BeanUtil.copyProperties(baseinfo,GovBizProjectBaseinfoDTO.class));
        return BizConst.OP_SUCCESS;
    }

    private void checkName(String baseProjName, String baseProjId) {
        VUtils.isTrue(baseinfoService.count(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjName,baseProjName)
                .ne(GovBizProjectBaseinfo::getBaseProjId,baseProjId)) > 0L)
                .throwMessage("项目名重复");
    }

    /**
     * 推送项目
     * @param saveDTO
     */
    private void pushProject(GovBizProjectSaveDTO saveDTO) {
        String url = BizProjectContant.PUSH_URL;
        ProjectPushReq req = ProjectConvert.convert(saveDTO);
        try{
            ResponseEntity<String> response = restTemplate.postForEntity(url, null, String.class, req);
            log.info("推送结果  {} :{}",saveDTO.getBaseProjId(),response);
        }catch (Exception e){
            log.info("推送失败,{}",e.getMessage());
        }
    }

    /**
     * 把项目置为无效
     * @param baseinfo
     */
    private void notEffectiveProject(GovBizProjectBaseinfoDTO baseinfo) {
        String url = BizProjectContant.NOT_EFFECTIVE_URL;
        baseinfo.setIsEffective(BizProjectContant.NOT_EFFECTIVE_CODE);
        ProjectBaseInfoReq req = ProjectConvert.convertBase(baseinfo);
        try{
            ResponseEntity<String> response = restTemplate.postForEntity(url, null, String.class, req);
            log.info("推送结果  {} :{}",baseinfo.getBaseProjId(),response);
        }catch (Exception e){
            log.info("推送失败,{}",e.getMessage());
        }
    }
}
