package com.ningdatech.syndataapi.scheduler.task;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.List;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.ningdatech.syndataapi.scheduler.model.entity.*;
import com.ningdatech.syndataapi.scheduler.service.*;

import cn.hutool.core.collection.CollUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * @author CMM
 * @since 2023/08/22 18:16
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class SynDataTask {

    private final IProjectApplyService projectApplyService;

    private final IProjectApproveService projectApproveService;

    private final IProjectBaseInfoService projectBaseInfoService;

    private final IProjectCimplementService projectCimplementService;

    private final IProjectMimplementService projectMimplementService;

    private final IProjectProcureService projectProcureService;

    @Value("${task.switch.is-open}")
    private boolean flag;

    @Value("${task.host}")
    public String HOST;

    // 测试数据
    private static final String PROJECT_BASE_INFO_TEST_DATA = "{\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseAreaName\":\"遂昌县\",\n" +
            "    \"baseAreaCode\":\"331123000\",\n" +
            "    \"baseProjIsConfidentiality\":\"01\",\n" +
            "    \"baseProjType\":\"01\",\n" +
            "    \"baseConstructionType\":\"01\",\n" +
            "    \"baseProjSetProg\":\"05\",\n" +
            "    \"baseProvManDeprt\":\"遂昌县大数据和金融发展中心\",\n" +
            "    \"baseProvManDeprtDing\":\"GO_9a7dce7fbaf54787998fa004f1b7c0e1\",\n" +
            "    \"baseManDeprt\":\"遂昌县大数据和金融发展中心\",\n" +
            "    \"baseManDeprtDing\":\"GO_9a7dce7fbaf54787998fa004f1b7c0e1\",\n" +
            "    \"baseManDepartUsci\":\"12332527MB1D48606A\",\n" +
            "    \"baseBuildDeprt\":\"遂昌县大数据和金融发展中心\",\n" +
            "    \"baseBuildDeprtDing\":\"GO_9a7dce7fbaf54787998fa004f1b7c0e1\",\n" +
            "    \"baseBuildDepartUsci\":\"12332527MB1D48606A\",\n" +
            "    \"baseProjPrincipal\":\"蒋国海\",\n" +
            "    \"baseProjPrincipalCall\":\"13884366607\",\n" +
            "    \"baseProjContacts\":\"毛以林\",\n" +
            "    \"baseProjContactsCall\":\"13857049999\"\n" +
            "}";


    private static final String PROJECT_APPLY_TEST_DATA = "{\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseDevelopCode\":\"否\",\n" +
            "    \"setProjCodeFinan\":\"331123220130969000000\",\n" +
            "    \"baseHistorProjName\":\"无\",\n" +
            "    \"baseHistorProjId\":\"000000\",\n" +
            "    \"baseAccountAppName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseBrainName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseProjStartTime\":\"2023—03—08\",\n" +
            "    \"baseProjEndTime\":\"2023—12—08\",\n" +
            "    \"baseProjDuration\":\"9月\",\n" +
            "    \"baseProjSetYear\":\"2023\",\n" +
            "    \"baseProjAmountOri\":\"02\",\n" +
            "    \"baseBasisAmountOri\":\"\",\n" +
            "    \"baseProjTotalAmount\":\"300\",\n" +
            "    \"baseProjDeclAmount\":\"300\",\n" +
            "    \"baseProjConsClass\":\"c\",\n" +
            "    \"baseLowestLevel\":\"c\",\n" +
            "    \"baseProjBasis\":\"01:02：03\",\n" +
            "    \"baseBasisEstablish\":\"丽水市中共遂昌县委组织部新时代挺进师战斗力指数应用解决了县级干部考评普遍存在考评指标模糊、展示界面缺乏、考核过程不规范、考评流程繁杂等问题，采用C/S架构，即客户端/服务器架构，包含五维考评、挂帅争先、村级红黑榜、天工考评、交办工作等考评模块，面向领导干部、乡干部、村干部等三类人员，实现了链条、全过程追溯每个班子、每名干部实绩的功能。\",\n" +
            "    \"baseProjBasisFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseProjIntro\":\"丽水市中共遂昌县委组织部新时代挺进师战斗力指数应用解决了县级干部考评普遍存在考评指标模糊、展示界面缺乏、考核过程不规范、考评流程繁杂等问题，采用C/S架构，即客户端/服务器架构，包含五维考评、挂帅争先、村级红黑榜、天工考评、交办工作等考评模块，面向领导干部、乡干部、村干部等三类人员，实现了链条、全过程追溯每个班子、每名干部实绩的功能。\",\n" +
            "    \"beseExpectedResults\":\"省项管系统项目的主要内容为项目全生命周期管理，业务对象为全省电子政务项目，预期成效指标有 项目数据归集覆盖11个地市和90个区县和项目信息完整率达98%以上\",\n" +
            "    \"baseResearchReportFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseProjApplyFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseOperatMaintenFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseProjOtherFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseProjRemark\":\"丽水市中共遂昌县委组织部新时代挺进师战斗力指数应用解决了县级干部考评普遍存在考评指标模糊、展示界面缺乏、考核过程不规范、考评流程繁杂等问题，采用C/S架构，即客户端/服务器架构，包含五维考评、挂帅争先、村级红黑榜、天工考评、交办工作等考评模块，面向领导干部、乡干部、村干部等三类人员，实现了链条、全过程追溯每个班子、每名干部实绩的功能。\"\n" +
            "}";

    private static final String PROJECT_APPROVE_TEST_DATA = "{\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseDevelopCode\":\"否\",\n" +
            "    \"setProjCodeFinan\":\"331123220130969000000\",\n" +
            "    \"baseReviewResults\":\"01\",\n" +
            "    \"baseReviewOpinion\":\"通过\",\n" +
            "    \"baseReviewCommentsFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseExpertTotalMoney\":\"300\",\n" +
            "    \"baseExpertYearMoney\":\"300\",\n" +
            "    \"approvalFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseInitialReviewTotalMoney\":\"300\",\n" +
            "    \"baseProjReplyAmount\":\"300\",\n" +
            "    \"equalProtectionLevel\":\"03\",\n" +
            "    \"preliminaryDesignScheme\":\"否\",\n" +
            "    \"preliminaryDesignFile\":\"否\",\n" +
            "    \"releaseYearMoney\":\"300\"\n" +
            "}";

    private static final String PROJECT_PROCURE_TEST_DATA = "{\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"setProjCodeFinan\":\"331123220130969000000\",\n" +
            "    \"baseBidCode\":\"33112322013096900000—01\",\n" +
            "    \"baseBidName\":\"标段一\",\n" +
            "    \"baseProjPurchaseWay\":\"01\",\n" +
            "    \"basePurchaseCode\":\"否\",\n" +
            "    \"purchaseFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"basePurchasingAgencies\":\"测试代理机构\",\n" +
            "    \"baseUnifiedCreditCode\":\"91330108MA2H2JUB3G\",\n" +
            "    \"baseWinningBidTime\":\"2022—03—08\",\n" +
            "    \"baseProjPurchaseAmount\":\"300\",\n" +
            "    \"biddingFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseConsDeprt\":\"测试供应商\",\n" +
            "    \"baseConsDeprtUsci\":\"91330108MA2H2JUB3G\",\n" +
            "    \"purchaseContract\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"basePaymentTime\":\"2023—03—08\",\n" +
            "    \"paymentProgress\":\"300\"\n" +
            "}";

    private static final String PROJECT_CIMPLEMENT_TEST_DATA = "{\n" +
            "    \"baseBidCode\":\"33112322013096900000—01\",\n" +
            "    \"baseBidName\":\"标段一\",\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseDevelopCode\":\"否\",\n" +
            "    \"baseProjSysCode\":\"\",\n" +
            "    \"baseInitialOpinionFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseInforLevelFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"basePasswAssessFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseThirdAcceptFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseCheckFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseFinanlAuditFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseUserConsFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseEstaSummFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseIrsTestRunFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseSummReportFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseLogAggregation\":\"2\",\n" +
            "    \"baseBusinessMetrics\":\"核心业务1+实际成效指标1+100\",\n" +
            "    \"baseFinalExpertOpinionFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseEngineerPostpoFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseChangeFormFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseChanFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"areaCode\":\"331100\"\n" +
            "}";

    private static final String PROJECT_MIMPLEMENT_TEST_DATA = "{\n" +
            "    \"baseBidCode\":\"33112322013096900000—01\",\n" +
            "    \"baseBidName\":\"标段一\",\n" +
            "    \"baseProjId\":\"33112322013096900000\",\n" +
            "    \"baseProjName\":\"遂昌县大数据和金融发展中心天工系统\",\n" +
            "    \"baseDevelopCode\":\"否\",\n" +
            "    \"baseProjSysCode\":\"\",\n" +
            "    \"baseInforLevelFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"basePasswAssessFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseThirdAcceptFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseUserConsFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseEstaSummFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseOperatMaintenSummFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseLogAggregation\":\"1\",\n" +
            "    \"baseBusinessMetrics\":\"核心业务1+实际成效指标1+100\",\n" +
            "    \"baseFinalExpertOpinionFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseEngineerPostpoFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseEngineerAlterFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"baseChanFile\":\"http://60.188.225.145/pm/file/download?fileId=951\",\n" +
            "    \"areaCode\":\"331100\"\n" +
            "}";



    /**
     * 前置机每天晚上10点自动清空，第二天早上6点获取数据
     * 定时同步前置机数据 每天1点开始执行一次
     *
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void doTask() throws UnknownHostException {
        if (flag && HOST.equals(InetAddress.getLocalHost().getHostName())) {
            log.info("数据同步任务开始");
            // 测试数据只有一条，这里只插入一条

            // 项目基本信息
            ProjectBaseInfo projectBaseInfo = JSON.parseObject(PROJECT_BASE_INFO_TEST_DATA, ProjectBaseInfo.class);
            projectBaseInfoService.save(projectBaseInfo);

            // 项目申报信息
            ProjectApply projectApply = JSON.parseObject(PROJECT_APPLY_TEST_DATA, ProjectApply.class);
            projectApplyService.save(projectApply);

            // 项目立项评审信息
            ProjectApprove projectApprove = JSON.parseObject(PROJECT_APPROVE_TEST_DATA, ProjectApprove.class);
            projectApproveService.save(projectApprove);

            // 项目采购信息
            ProjectProcure projectProcure = JSON.parseObject(PROJECT_PROCURE_TEST_DATA, ProjectProcure.class);
            projectProcureService.save(projectProcure);

            // 建设项目实施信息
            ProjectCimplement projectCimplement = JSON.parseObject(PROJECT_CIMPLEMENT_TEST_DATA, ProjectCimplement.class);
            projectCimplementService.save(projectCimplement);

            // 运维项目实施信息
            ProjectMimplement projectMimplement = JSON.parseObject(PROJECT_MIMPLEMENT_TEST_DATA, ProjectMimplement.class);
            projectMimplementService.save(projectMimplement);
            log.info("数据同步任务结束");
        }
    }
}
